(function () {
    const PluginDudenToggleButton = (props) => {
        if (document.querySelector('.dkce-toggle-button'))
            return;

        if (!document.body.classList.contains('block-editor-page'))
            return;

        var dkc_initialized = false;

        let settings = JSON.parse(JSON.stringify(DKCE_SETTINGS));
        settings.overlayMode = true;
        settings.setupWatcher = false;
        settings.activated = (localStorage.getItem('gutenberg.duden.activated') === 'true');
        
        const dkceIcon = document.createElement('img');
        dkceIcon.src = settings.path + 'assets/icon_32.png';
        dkceIcon.style.borderRadius = '2px';

        const dkceButton = document.createElement('button');
        dkceButton.classList.add('components-button', 'has-icon', 'dkce-toggle-button');
        dkceButton.type = "button";
        dkceButton.appendChild(dkceIcon);

        const setButtonStyle = (isHovered, isActive) => {
            const css = dkceButton.style;
            
            css.borderRadius = '2px';
            css.backgroundColor = 'inherit';
            css.filter = 'grayscale(100%)'
            css.webkitFilter = 'grayscale(100%)'
            css.opacity = 0.5;

            if (isHovered) {
                css.backgroundColor = '#017cba';
            }

            if (isHovered || isActive) {
                css.opacity = 1.0;
                css.filter = ''
                css.webkitFilter = ''
            }
        };

        dkceButton.addEventListener('mouseover', () => {
            setButtonStyle(true, settings.activated);
        });

        dkceButton.addEventListener('mouseleave', () => {
            setButtonStyle(false, settings.activated);
        });

        dkceButton.addEventListener('click', () => {
            if (dkc_initialized) {
                settings.activated = !settings.activated;
                const message = {
                    subject: 'activate',
                    activate: settings.activated
                }
        
                localStorage.setItem('gutenberg.duden.activated', settings.activated);
                window.postMessage(message, window);                
            }

            setButtonStyle(false, settings.activated);
        });

        const wpPublishButton = document.querySelector('.editor-post-publish-button__button, .editor-post-publish-button');
        if (!wpPublishButton) {
            console.warn("DKCE could not be initialized because could not find Gutenberg Publish Button. Try again in 1 second.");
            window.setTimeout(PluginDudenToggleButton, 1000);
            return;
        }        
        wpPublishButton.parentNode.insertBefore(dkceButton, wpPublishButton.nextSibling);

        setButtonStyle(false, settings.activated);
        
        ignoreElement = (element) => {
            return (!element.isContentEditable) || (element.hasAttribute('dkceSpellcheckerId'));
        };

        createSpellchecker = (element) => {
            DKCE.init(element, settings, () => {
                dkc_initialized = true;
            });
        };

        checkForNewBlocks = () => {            
            var root = document;
            var iFrameCanvas = document.getElementsByName("editor-canvas");
            if (iFrameCanvas.length > 0 && iFrameCanvas[0].tagName == 'IFRAME') 
                root = iFrameCanvas[0].contentWindow.document.body;
            var elementsToCheck = root.querySelectorAll(".rich-text,.block-library-rich-text__tinymce");
            for (var i = 0; i < elementsToCheck.length; ++i) {
                var element = elementsToCheck[i];
                if (ignoreElement(element)) continue;
                createSpellchecker(element);
            }
            setTimeout(checkForNewBlocks, 1000);
        }
       
        window._wpLoadBlockEditor.then(() => {
            setTimeout(checkForNewBlocks, 100);
        });
    };
   
    window.addEventListener('load',() => {
        wp.plugins.registerPlugin('plugin-document-setting-panel-duden', {
               render: PluginDudenToggleButton
        });
    });
})();